#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import pathlib
import re
import sys

import pytest

from youqu3 import logger, setting


class Run:
    """
    本地执行器
    """

    __author__ = "mikigo <huangmingqiang@uniontech.com>"

    def __init__(
            self,
            keywords=None,
            tags=None,
            **kwargs,
    ):
        logger("INFO")

        self.keywords = keywords
        self.tags = tags
        self.rootdir = pathlib.Path(".").absolute()
        self.report_path = self.rootdir / "report"
        self.html_report_path = self.report_path / "html"
        self.allure_data_path = self.html_report_path / "_data"
        self.allure_html_path = self.html_report_path / "html"
        self.json_report_path = self.report_path / "json"

    @staticmethod
    def makedirs(dirs):
        pathlib.Path(dirs).mkdir(parents=True, exist_ok=True)

    @staticmethod
    def set_recursion_limit(strings):
        len_tags = len(re.split("or |and |not ", strings))
        if len_tags >= 999:
            sys.setrecursionlimit(len_tags + 100)

    def generate_cmd(self):
        cmd = ["pytest"]

        if self.keywords:
            self.set_recursion_limit(self.keywords)
            cmd.extend(["-k", f"'{self.keywords}'"])
        if self.tags:
            self.set_recursion_limit(self.tags)
            cmd.extend(["-m", f"'{self.tags}'"])

        cmd.extend([
            f"--maxfail={setting.MAX_FAIL}",
            f"--reruns={setting.RERUNS}",
            f"--timeout={setting.TIMEOUT}",
            "--json-report",
            "--json-report-indent=2",
            f"--json-report-file={self.json_report_path / f'report_{setting.TIME_STRING}.json'}",
            f"--alluredir={self.allure_data_path}",
            "--clean-alluredir",
        ])

        return cmd

    def run(self):
        pytest.main(
            [i.strip("'") for i in self.generate_cmd()[1:]]
        )

        from allure_custom import AllureCustom
        AllureCustom.gen(str(self.allure_data_path), str(self.allure_html_path), clean=True)


if __name__ == "__main__":
    Run().run()



import click
from rich.console import Console

console = Console()


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    from youqu3 import version
    click.echo(version)
    ctx.exit()


@click.group()
def cli():
    ...


@cli.command()
@click.option("-k", "--keywords", default=None, type=click.STRING, help="keywords driver")
@click.option("-t", "--tags", default=None, type=click.STRING, help="tags driver")
def run(
        keywords,
        tags,
):
    """RUN驱动"""
    args = {
        "keywords": keywords,
        "tags": tags,
    }
    from youqu3.driver.run import Run
    Run(**args).run()


@cli.command()
@click.option("-k", "--keywords", default=None, type=click.STRING, help="keywords driver")
@click.option("-t", "--tags", default=None, type=click.STRING, help="tags driver")
@click.option("-c", "--clients", default=None, type=click.STRING, help="clients")
@click.option("-s", "--send", default=None, type=click.STRING, help="send code")
@click.option("-e", "--build-env", default=None, type=click.STRING, help="buildenv")
@click.option("-m", "--mode", default="parallel", type=click.Choice(["parallel", "nginx"]), help="mode")
def remote(
        keywords,
        tags,
        clients,
        send,
        build_env,
        mode,
):
    """REMOTE驱动"""
    args = {
        "keywords": keywords,
        "tags": tags,
        "clients": clients,
        "send": send,
        "build_env": build_env,
        "mode": mode,
    }
    from youqu3.driver.remote import Remote
    Remote(**args).run()


@cli.command()
def init():
    """创建用例工程"""
    from youqu3.driver.init import Init
    Init().init()


@cli.command()
def env():
    """安装YouQu环境"""
    from youqu3.driver.env import env
    env()


if __name__ == '__main__':
    cli()



import dataclasses
import enum

from youqu3._setting._dynamic import _DynamicSetting


class _Setting(_DynamicSetting):
    """Global Config"""

    PASSWORD: str = "1"

    MAX_FAIL = 1
    TIMEOUT = 300
    LOG_LEVEL = "INFO"
    RERUNS = 1
    RECORD_FAILED_CASE = 1

    # OCR
    OCR_NETWORK_RETRY = 1
    OCR_PAUSE = 1
    OCR_TIMEOUT = 5
    OCR_MAX_MATCH_NUMBER = 100
    OCR_PORT = 8890
    OCR_SERVER_HOST = "10.8.13.7/10.8.13.66/10.8.13.55/10.8.13.100"

    # IMAGE
    IMAGE_NETWORK_RETRY = 1
    IMAGE_PAUSE = 1
    IMAGE_TIMEOUT = 5
    IMAGE_MAX_MATCH_NUMBER = 100
    IMAGE_PORT = 8889
    IMAGE_SERVER_HOST = "10.8.11.139"

    # REMOTE
    MODE = "parallel"
    BUILD_ENV = True
    SEND = True

    #
    PYPI_MIRROR = "https://pypi.tuna.tsinghua.edu.cn/simple"

    @dataclasses.dataclass
    class Sleepx:
        x86_64: [float, int] = 1
        aarch64: [float, int] = 1.5
        loongarch64: [float, int] = 2
        mips64: [float, int] = 2.5
        sw64: [float, int] = 2.5

    @enum.unique
    class FixedCsvTitle(enum.Enum):
        case_id = "脚本ID"
        skip_reason = "跳过原因"
        fixed = "确认修复"
        removed = "废弃用例"


setting = _Setting()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import pathlib
import re
import time
from concurrent.futures import ALL_COMPLETED
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import wait

from allure_custom import AllureCustom

from youqu3 import logger
from youqu3 import setting
from youqu3 import sleep
from youqu3.cmd import Cmd, RemoteCmd


class Remote:
    __author__ = "mikigo <huangmingqiang@uniontech.com>"

    def __init__(
            self,
            keywords=None,
            tags=None,
            clients=None,
            send=None,
            build_env=None,
            mode=None,
            **kwargs
    ):
        logger("INFO")
        self.keywords = keywords
        self.tags = tags
        self.clients = clients
        self.send = send or setting.SEND
        self.build_env = build_env or setting.BUILD_ENV
        self.mode = mode or setting.MODE

        cli_clients = {}
        if self.clients:
            _cli_clients = self.clients.split("/")
            for index, _client in enumerate(_cli_clients):
                _cli_client_info = re.findall(r"^(.+?)@(\d+\.\d+\.\d+\.\d+):{0,1}(.*?)$", _client)
                if _cli_client_info:
                    _c = list(_cli_client_info[0])
                    if _c[2] == "":
                        _c[2] = setting.PASSWORD
                    cli_clients[f"client{index + 1}"] = _c

        else:
            raise ValueError("REMOTE驱动模式, 未传入远程客户端信息：-c/--clients user@ip:pwd")

        self.clients = cli_clients

        self.server_rootdir = pathlib.Path(".").absolute()
        self.rootdir_name = self.server_rootdir.name

        self.client_rootdir = lambda x: f"/home/{x}/{self.rootdir_name}_{setting.TIME_STRING}"
        self.client_report_path = lambda x: f"{self.client_rootdir(x)}/report"
        self.client_html_report_path = lambda x: f"{self.client_report_path(x)}/html"
        self.client_json_report_path = lambda x: f"{self.client_report_path(x)}/json"

        self.strf_time = time.strftime("%m%d%p%I%M%S")
        self.rsync = "sshpass -p '%s' rsync -av -e ssh"
        self.empty = "> /dev/null 2>&1"

        self.collection_json = False
        self.server_json_dir_id = None
        self.pms_user = None
        self.pms_password = None

    def send_code(self, user, _ip, password):
        logger.info(f"开始发送代码到测试机 - < {user}@{_ip} >")
        RemoteCmd(user, _ip, password).remote_sudo_run(f"rm -rf {self.client_rootdir(user)} {self.empty}")
        RemoteCmd(user, _ip, password).remote_run(f"mkdir -p {self.client_rootdir(user)} {self.empty}")
        exclude = ""
        for i in [
            "__pycache__",
            ".pytest_cache",
            ".vscode",
            ".idea",
            ".git",
            ".github",
            ".venv",
            "report",
            ".gitignore",
            "LICENSE",
            "Pipfile",
            "Pipfile.lock",
            "README.md",
        ]:
            exclude += f"--exclude='{i}' "
        stdout = Cmd.run(
            f"{self.rsync % (password,)} {exclude} {self.server_rootdir}/* {user}@{_ip}:{self.client_rootdir(user)}/ && echo 0 || echo 1",
        )
        logger.info(f"代码发送{'成功' if stdout == 0 else '失败'} - < {user}@{_ip} >")

    def install_client_env(self, user, _ip, password):
        logger.info(f"开始安装环境 - < {user}@{_ip} >")

        def _env(cmd):
            return RemoteCmd(user, _ip, password).remote_run(cmd)

        _env(f"pip3 install youqu3 -i {setting.PYPI_MIRROR}")
        result = _env(f"export PATH=$PATH:$HOME/.local/bin;cd {self.client_rootdir(user)} && youqu3 env && echo 0 || echo 1")
        logger.info(f"环境安装{'成功' if result == 0 else '失败'} - < {user}@{_ip} >")

    def send_code_and_env(self, user, _ip, password):
        self.send_code(user, _ip, password)
        self.install_client_env(user, _ip, password)

    @staticmethod
    def makedirs(dirs):
        pathlib.Path(dirs).mkdir(parents=True, exist_ok=True)

    def get_back_all_report(self, client_list):
        def get_back(user, _ip, password):
            server_html_path = f"{self.server_rootdir}/report/remote/{self.strf_time}_{_ip}_{self.rootdir_name}"
            self.makedirs(server_html_path)
            Cmd.run(
                f"{self.rsync % password} {user}@{_ip}:{self.client_report_path(user)}/* {server_html_path}/ {self.empty}")

        if len(self.clients) >= 2:
            _ps = []
            executor = ThreadPoolExecutor()
            for client in client_list[:-1]:
                user, _ip, password = self.clients.get(client)
                _p4 = executor.submit(get_back, user, _ip, password)
                _ps.append(_p4)
                sleep(2)
            user, _ip, password = self.clients.get(client_list[-1])
            get_back(user, _ip, password)
            wait(_ps, return_when=ALL_COMPLETED)
        else:
            user, _ip, password = self.clients.get(client_list[0])
            get_back(user, _ip, password)

    def generate_remote_cmd(self, user):
        cmd = ["cd", f"{self.client_rootdir(user)}/", "&&", "youqu3", "run"]

        if self.keywords:
            cmd.extend(["-k", f"'{self.keywords}'"])
        if self.tags:
            cmd.extend(["-m", f"'{self.tags}'"])

        cmd.extend([
            f"--maxfail={setting.MAX_FAIL}",
            f"--reruns={setting.RERUNS}",
            f"--timeout={setting.TIMEOUT}",
            "--json-report",
            "--json-report-indent=2",
            f"--json-report-file={self.client_json_report_path(user)}/report_{setting.TIME_STRING}.json",
            f"--alluredir={self.client_html_report_path(user)}",
        ])

        return cmd

    def run_test(self, user, _ip, password):
        RemoteCmd(user, _ip, password).remote_run(" ".join(self.generate_remote_cmd(user)))

    def parallel_run(self):
        _ps = []
        executor = ThreadPoolExecutor()
        for client in list(self.clients.keys())[:-1]:
            user, _ip, password = self.clients.get(client)
            _p3 = executor.submit(self.run_test, user, _ip, password)
            _ps.append(_p3)
            sleep(1)
        user, _ip, password = list(self.clients.values())[-1]
        self.run_test(user, _ip, password)
        wait(_ps, return_when=ALL_COMPLETED)
        sleep(5)

    def mul_do(self, func_obj, client_list):
        if len(client_list) >= 2:
            executor = ThreadPoolExecutor()
            _ps = []
            for client in client_list[:-1]:
                user, _ip, password = self.clients.get(client)
                _p1 = executor.submit(func_obj, user, _ip, password)
                _ps.append(_p1)
            user, _ip, password = self.clients.get(client_list[-1])
            func_obj(user, _ip, password)
            wait(_ps, return_when=ALL_COMPLETED)
        else:
            user, _ip, password = self.clients.get(client_list[0])
            func_obj(user, _ip, password)

    def run(self):
        client_list = list(self.clients.keys())
        from rich.console import Console
        from rich.table import Table

        console = Console()
        table = Table(
            title="远程测试机列表",
            title_style="strike",
            show_header=True,
            header_style="bold magenta",
            highlight=True,
        )
        table.add_column("CLIENTS", justify="center")
        table.add_column("USER", justify="center")
        table.add_column("IP", justify="center")
        table.add_column("PASSWORD", justify="center")
        for c, (user, _ip, password) in self.clients.items():
            table.add_row(c, user, _ip, password)
        console.print(table)
        if self.build_env:
            self.mul_do(self.send_code_and_env, client_list)
        else:
            if self.send:
                self.mul_do(self.send_code, client_list)

        if self.mode == "parallel":
            self.parallel_run()
        else:
            ...
        self.get_back_all_report(client_list)



import pathlib
from youqu3.cmd import Cmd
from youqu3 import exception
from youqu3 import setting


def env():
    rootdir = pathlib.Path(".").absolute()
    try:
        Cmd.run("pip3 --version", interrupt=True)
    except exception.ShellExecutionFailed:
        Cmd.run('curl -sSL https://bootstrap.pypa.io/get-pip.py -o get-pip.py')

    Cmd.run(f'pip3 install youqu3 -i {setting.PYPI_MIRROR}')
    Cmd.run(f'pip3 install pipenv -i {setting.PYPI_MIRROR}')
    Cmd.run("pipenv --python 3")
    Cmd.run(f"pipenv run pip install -r {rootdir}/requirements.txt")


if __name__ == '__main__':
    env()



import random

from youqu3.exception import YouQuPluginInstalledError

try:
    from youqu_imagecenter_rpc import ImageCenter as _ImageCenter
    from youqu_imagecenter_rpc import setting as image_setting

    from youqu3 import setting

    HAS_IMAGECENTER = False
except ImportError:
    HAS_IMAGECENTER = False

if HAS_IMAGECENTER is False:
    raise YouQuPluginInstalledError("youqu-imagecenter-rpc")

from youqu3.mkmixin import MouseKeyChainMixin


class ImageCenter(MouseKeyChainMixin):
    image_setting.NETWORK_RETRY = int(setting.IMAGE_NETWORK_RETRY)
    image_setting.PAUSE = float(setting.IMAGE_PAUSE)
    image_setting.TIMEOUT = float(setting.IMAGE_TIMEOUT)
    image_setting.MAX_MATCH_NUMBER = int(setting.IMAGE_MAX_MATCH_NUMBER)
    image_setting.PORT = setting.IMAGE_PORT
    _image_servers = [i.strip() for i in setting.IMAGE_SERVER_HOST.split("/") if i]
    x = None
    y = None
    result = None

    @classmethod
    def find_image(cls, *args, **kwargs):
        servers = cls._image_servers
        while servers:
            image_setting.SERVER_IP = random.choice(servers)
            if _ImageCenter.check_connected() is False:
                servers.remove(image_setting.SERVER_IP)
                image_setting.SERVER_IP = None
            else:
                break
        if image_setting.SERVER_IP is None:
            raise EnvironmentError(f"所有IMAGE服务器不可用: {cls._image_servers}")
        cls.result = _ImageCenter.find_image(*args, **kwargs)

        if isinstance(cls.result, tuple):
            cls.x, cls.y = cls.result

        return cls



from youqu3.exception import YouQuPluginInstalledError

try:
    from youqu_mousekey import MouseKey as MouseKey

    HAS_MOUSEKEY = True
except ImportError:
    HAS_MOUSEKEY = False

if HAS_MOUSEKEY is False:
    raise YouQuPluginInstalledError("youqu-mousekey")



import subprocess
import sys

from youqu3 import exception
from youqu3 import logger
from youqu3 import setting


class Cmd:

    @staticmethod
    def _run(command, _input=None, timeout=None, check=False, **kwargs):
        with subprocess.Popen(command, **kwargs) as process:
            try:
                stdout, stderr = process.communicate(_input, timeout=timeout)
            except:  # Including KeyboardInterrupt, communicate handled that.
                process.kill()
                raise
            retcode = process.poll()
            if check and retcode:
                raise subprocess.CalledProcessError(
                    retcode, process.args, output=stdout, stderr=stderr
                )
        return subprocess.CompletedProcess(process.args, retcode, stdout, stderr)

    @classmethod
    def _getstatusoutput(cls, command, timeout):
        kwargs = {
            "shell": True,
            "stderr": subprocess.STDOUT,
            "stdout": subprocess.PIPE,
            "timeout": timeout,
        }
        try:
            if sys.version_info >= (3, 7):
                kwargs["text"] = True
            result = cls._run(command, **kwargs)
            data = result.stdout
            if isinstance(data, bytes):
                data = data.decode("utf-8")
            exitcode = result.returncode
        except subprocess.CalledProcessError as ex:
            data = ex.output
            exitcode = ex.returncode
        except subprocess.TimeoutExpired as ex:
            # pylint: disable=unnecessary-dunder-call
            data = ex.__str__()
            exitcode = -1
        if data[-1:] == "\n":
            data = data[:-1]
        return exitcode, data

    @classmethod
    def sudo_run(
            cls,
            command,
            interrupt=False,
            timeout=25,
            print_log=True,
            command_log=True,
            password=None,
    ):
        if password is None:
            password = setting.PASSWORD
        return cls.run(
            f"echo '{password}' | sudo -S {command}",
            interrupt=interrupt,
            timeout=timeout,
            print_log=print_log,
            command_log=command_log,
        )

    @classmethod
    def run(cls, command, interrupt=False, timeout=25, print_log=True, command_log=True):
        """
         执行shell命令
        :param command: shell 命令
        :param interrupt: 命令异常时是否中断
        :param timeout: 命令执行超时
        :param out_debug_flag: 命令返回信息输出日志
        :param command_log: 执行的命令字符串日志
        :return: 返回终端输出
        """
        status, out = cls._getstatusoutput(command, timeout=timeout)
        if command_log:
            logger.debug(command)
        if status != 0 and interrupt:
            raise exception.ShellExecutionFailed(out)
        if print_log and out:
            logger.debug(out)
        return out


class RemoteCmd:

    def __init__(self, user: str, ip: str, password: str):
        self.user = user
        self.ip = ip
        self.password = password

    def remote_run(self, cmd: str):
        try:
            from fabric import Connection
        except ImportError:
            raise exception.YouQuPluginInstalledError("fabric")
        c = Connection(host=self.ip, user=self.user, connect_kwargs={'password': self.password})
        res = c.run(cmd)
        return res.stdout

    def remote_sudo_run(self, cmd: str):
        try:
            from fabric import Connection, Config
        except ImportError:
            raise exception.YouQuPluginInstalledError("fabric")
        c = Connection(
            host=self.ip, user=self.user, connect_kwargs={'password': self.password},
            config=Config(overrides={'sudo': {'password': self.password}})
        )
        res = c.sudo(cmd)
        return res.stdout


if __name__ == '__main__':
    stdout = Cmd.run(
        "sshpass -p '1' rsync -av -e ssh --exclude='__pycache__' --exclude='.pytest_cache' --exclude='.vscode' --exclude='.idea' --exclude='.git' --exclude='.github' --exclude='.venv' --exclude='report' --exclude='.gitignore'  /home/mikigo/github/youqu3-testcase/* uos@10.8.7.55:/home/uos/youqu3-testcase/",
        interrupt=True,
        print_log=False,
    )
    print(stdout)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib
import re
import shutil
from time import strftime

from rich import print
from rich.filesize import decimal
from rich.markup import escape
from rich.text import Text
from rich.tree import Tree

from youqu3 import setting


def walk_directory(directory: pathlib.Path, tree: Tree) -> None:
    """Recursively build a Tree with directory contents."""
    # Sort dirs first then by filename
    paths = sorted(
        pathlib.Path(directory).iterdir(),
        key=lambda path: (path.is_file(), path.name.lower()),
    )
    for path in paths:
        # Remove hidden files
        if path.name.startswith("."):
            continue
        if path.is_dir():
            style = "dim" if path.name.startswith("__") else ""
            branch = tree.add(
                f"[bold magenta]:open_file_folder: [link file://{path}]{escape(path.name)}",
                style=style,
                guide_style=style,
            )
            walk_directory(path, branch)
        else:
            text_filename = Text(path.name, "green")
            text_filename.highlight_regex(r"\..*$", "bold red")
            text_filename.stylize(f"link file://{path}")
            file_size = path.stat().st_size
            text_filename.append(f" ({decimal(file_size)})", "blue")
            icon = "🐍 " if path.suffix == ".py" else "📄 "
            tree.add(Text(icon) + text_filename)


class Init:

    @staticmethod
    def copy_template_to_apps():
        # if os.listdir("."):
        #     if input("当前目录存在且里面存在文件，请确认是否清空（Y/N）：") in ("y", "Y"):
        #         os.system(f"rm -rf ./*")
        #     else:
        #         exit(0)
        os.system(f"cp -r {setting.TPL_PATH}/* .")
        os.system(f"cp {setting.TPL_PATH}/.gitignore-tpl  .")

    @classmethod
    def init(cls):
        cls.copy_template_to_apps()
        dirname = pathlib.Path(".").parent.name
        for root, dirs, files in os.walk("."):
            for file in files:
                app_name = dirname.lower()

                if file.endswith("-tpl"):

                    if "${app_name}" in file:
                        new_file = re.sub(
                            r"\${app_name}", app_name, file
                        )
                        shutil.move(f"{root}/{file}", f"{root}/{new_file}")
                        file = new_file

                    if ".py-tpl" in file or ".csv-tpl" in file:
                        with open(f"{root}/{file}", "r") as f:
                            codes = f.readlines()
                        new_codes = []
                        for code in codes:
                            if "${APP_NAME}" in code:
                                code = re.sub(r"\${APP_NAME}", app_name.upper(), code)
                            if "${app_name}" in code:
                                code = re.sub(r"\${app_name}", app_name, code)
                            if "${AppName}" in code:
                                code = re.sub(r"\${AppName}", app_name.title().replace("_", "").replace("-", ""), code)
                            if "${USER}" in code:
                                code = re.sub(r"\${USER}", setting.USERNAME, code)
                            if "${DATE}" in code:
                                code = re.sub(r"\${DATE}", strftime("%Y/%m/%d"), code)
                            if "${TIME}" in code:
                                code = re.sub(r"\${TIME}", strftime("%H:%M:%S"), code)
                            if "${FIXEDCSVTITLE}" in code:
                                code = re.sub(
                                    r"\${FIXEDCSVTITLE}",
                                    ",".join([i.value for i in setting.FixedCsvTitle]),
                                    code,
                                )
                            new_codes.append(code)
                        with open(f"{root}/{file}", "w") as f:
                            f.writelines([i for i in new_codes])

                    shutil.move(f"{root}/{file}", f"{root}/{file}".rstrip("-tpl"))

        tree = Tree(
            f":open_file_folder: [link file://{dirname}]{dirname}",
            guide_style="bold bright_blue",
        )
        walk_directory(pathlib.Path(dirname), tree)
        print(tree)



from youqu3.exception import YouQuPluginInstalledError

try:
    from youqu_button_center import ButtonCenter as _ButtonCenter

except ImportError:
    raise YouQuPluginInstalledError("youqu-button-center")

from youqu3.mkmixin import MouseKeyChainMixin


class UI(MouseKeyChainMixin):

    def __init__(
            self,
            appname: str,
            config_path: str,
            number: int = -1,
            pause: int = 1,
            retry: int = 1,
    ):
        self.appname = appname
        self.config_path = config_path
        self.number = number
        self.pause = pause
        self.retry = retry

    def ui(
            self,
            btn_name,
            offset_x=None,
            multiplier_x=None,
            offset_y=None,
            multiplier_y=None,
    ):
        MouseKeyChainMixin.result = _ButtonCenter(
            appname=self.appname,
            config_path=self.config_path,
            number=self.number,
            pause=self.pause,
            retry=self.retry
        ).btn_center(
            btn_name,
            offset_x=offset_x,
            multiplier_x=multiplier_x,
            offset_y=offset_y,
            multiplier_y=multiplier_y,
        )
        if isinstance(self.result, tuple):
            MouseKeyChainMixin.x, MouseKeyChainMixin.y = MouseKeyChainMixin.result
        return self

if __name__ == '__main__':
    a = UI(app_name="dde-file-manager", config_path="/home/mikigo/github/youqu3/ui.ini").ui("播放按钮").center()
    print(a)

